#ifndef IMDI_H
#define IMDI_H

/* Integer Multi-Dimensional Interpolation */

/*
 * Copyright 2000 - 2006 Graeme W. Gill
 * All rights reserved.
 *
 * This material is licenced under the GNU GENERAL PUBLIC LICENCE :-
 * see the Licence.txt file for licencing details.
 */

/*
 * This file provides the common definitions for IMDI, and
 * in particular, the runtime conversion object. 
 * Actual runtime details are kept opaque.
 */

#include "imdi_utl.h"
#include "imdi_arch.h"
#include "imdi_gen.h"

/* IMDI Object */
struct _imdi {
	void *impl;			/* Pointer to implementation information (type imdi_imp *) */

	/* Do the interpolation. */
	/* Each pointer corresponds to the colors plane for plane interleaved. */
	/* pointer[0] is used for pixel interleave. */
	/* stride is only obeyed if the appropriate option flag was set */
	/* in new_imdi, and is in pixel components, and effectively defaults */
	/* to 1 for plane interleaved, and id/od for pixel interleave. */
	void (*interp)(struct _imdi *s, void **outp, int outst,		/* Ouput pointers and stride */
	                                void **inp, int inst,		/* Input pointers and stride */
	                                unsigned int npixels);		/* Number of pixels */
	void (*del)(struct _imdi *s);		/* Delete this object */

}; typedef struct _imdi imdi;

/* Create a new imdi */
/* Return NULL if request is not supported */
imdi *new_imdi(
	int id,				/* Number of input dimensions */
	int od,				/* Number of output dimensions */
	imdi_pixrep in,		/* Input pixel representation */
	int in_signed,		/* Bit flag per channel, NZ if treat as signed */
	int *inm,			/* Input raster channel to callback channel mapping, NULL for none. */
	imdi_iprec iprec,	/* Internal processing precision */
	imdi_pixrep out,	/* Output pixel representation */
	int out_signed,		/* Bit flag per channel, NZ if treat as signed */
	int *outm,			/* Output raster channel to callback channel mapping, NULL for none. */
	int res,			/* Desired table resolution */
	imdi_options opt,	/* Direction and stride options */

	/* Callbacks to lookup the imdi table values */
	void (*input_curves) (void *cntx, double *out_vals, double *in_vals),
	void (*md_table)     (void *cntx, double *out_vals, double *in_vals),
	void (*output_curves)(void *cntx, double *out_vals, double *in_vals),
	void *cntx		/* Context to callbacks */
);

#endif /* IMDI_H */






